@@ -79,7 +79,7 @@ ostream& ObjBencher::out(ostream& os, utime_t& t)
 
 ostream& ObjBencher::out(ostream& os)
 {
-  utime_t cur_time = ceph_clock_now(g_ceph_context);
+  utime_t cur_time = ceph_clock_now(cct);
   return out(os, cur_time);
 }
 
@@ -90,13 +90,12 @@ void *ObjBencher::status_printer(void *_bencher) {
   int i = 0;
   int previous_writes = 0;
   int cycleSinceChange = 0;
-  double avg_bandwidth;
   double bandwidth;
   utime_t ONE_SECOND;
   ONE_SECOND.set_from_double(1.0);
   bencher->lock.Lock();
   while(!data.done) {
-    utime_t cur_time = ceph_clock_now(g_ceph_context);
+    utime_t cur_time = ceph_clock_now(bencher->cct);
 
     if (i % 20 == 0) {
       if (i > 0)
@@ -131,7 +130,7 @@ void *ObjBencher::status_printer(void *_bencher) {
       data.history.bandwidth.push_back(bandwidth);
     }
 
-    avg_bandwidth = (double) (data.trans_size) * (data.finished)
+    double avg_bandwidth = (double) (data.trans_size) * (data.finished)
       / (double)(cur_time - data.start_time) / (1024*1024);
     if (previous_writes != data.finished) {
       previous_writes = data.finished;
@@ -159,13 +158,16 @@ void *ObjBencher::status_printer(void *_bencher) {
     }
     ++i;
     ++cycleSinceChange;
-    cond.WaitInterval(g_ceph_context, bencher->lock, ONE_SECOND);
+    cond.WaitInterval(bencher->cct, bencher->lock, ONE_SECOND);
   }
   bencher->lock.Unlock();
   return NULL;
 }
 
-int ObjBencher::aio_bench(int operation, int secondsToRun, int concurrentios, int op_size, bool cleanup) {
+int ObjBencher::aio_bench(
+  int operation, int secondsToRun,
+  int maxObjectsToCreate,
+  int concurrentios, int op_size, bool cleanup) {
   int object_size = op_size;
   int num_objects = 0;
   char* contentsChars = new char[op_size];
@@ -204,7 +206,7 @@ int ObjBencher::aio_bench(int operation, int secondsToRun, int concurrentios, in
   sanitize_object_contents(&data, data.object_size);
 
   if (OP_WRITE == operation) {
-    r = write_bench(secondsToRun, concurrentios);
+    r = write_bench(secondsToRun, maxObjectsToCreate, concurrentios);
     if (r != 0) goto out;
   }
   else if (OP_SEQ_READ == operation) {
@@ -297,10 +299,15 @@ int ObjBencher::fetch_bench_metadata(const std::string& metadata_file, int* obje
   return 0;
 }
 
-int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
+int ObjBencher::write_bench(int secondsToRun, int maxObjectsToCreate,
+			    int concurrentios) {
+  if (maxObjectsToCreate > 0 && concurrentios > maxObjectsToCreate)
+    concurrentios = maxObjectsToCreate;
   out(cout) << "Maintaining " << concurrentios << " concurrent writes of "
-       << data.object_size << " bytes for at least "
-       << secondsToRun << " seconds." << std::endl;
+	    << data.object_size << " bytes for up to "
+	    << secondsToRun << " seconds or "
+	    << maxObjectsToCreate << " objects"
+	    << std::endl;
   bufferlist* newContents = 0;
 
   std::string prefix = generate_object_prefix();
@@ -332,10 +339,10 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
 
   pthread_create(&print_thread, NULL, ObjBencher::status_printer, (void *)this);
   lock.Lock();
-  data.start_time = ceph_clock_now(g_ceph_context);
+  data.start_time = ceph_clock_now(cct);
   lock.Unlock();
   for (int i = 0; i<concurrentios; ++i) {
-    start_times[i] = ceph_clock_now(g_ceph_context);
+    start_times[i] = ceph_clock_now(cct);
     r = create_completion(i, _aio_cb, (void *)&lc);
     if (r < 0)
       goto ERR;
@@ -357,8 +364,9 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
   runtime.set_from_double(secondsToRun);
   stopTime = data.start_time + runtime;
   slot = 0;
-  while( ceph_clock_now(g_ceph_context) < stopTime ) {
-    lock.Lock();
+  lock.Lock();
+  while( ceph_clock_now(cct) < stopTime &&
+	 (!maxObjectsToCreate || data.started < maxObjectsToCreate)) {
     bool found = false;
     while (1) {
       int old_slot = slot;
@@ -389,7 +397,7 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
       lock.Unlock();
       goto ERR;
     }
-    data.cur_latency = ceph_clock_now(g_ceph_context) - start_times[slot];
+    data.cur_latency = ceph_clock_now(cct) - start_times[slot];
     data.history.latency.push_back(data.cur_latency);
     total_latency += data.cur_latency;
     if( data.cur_latency > data.max_latency) data.max_latency = data.cur_latency;
@@ -399,11 +407,11 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
     --data.in_flight;
     lock.Unlock();
     release_completion(slot);
-    timePassed = ceph_clock_now(g_ceph_context) - data.start_time;
+    timePassed = ceph_clock_now(cct) - data.start_time;
 
     //write new stuff to backend, then delete old stuff
     //and save locations of new stuff for later deletion
-    start_times[slot] = ceph_clock_now(g_ceph_context);
+    start_times[slot] = ceph_clock_now(cct);
     r = create_completion(slot, _aio_cb, &lc);
     if (r < 0)
       goto ERR;
@@ -411,15 +419,15 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
     if (r < 0) {//naughty; doesn't clean up heap space.
       goto ERR;
     }
-    lock.Lock();
-    ++data.started;
-    ++data.in_flight;
-    lock.Unlock();
     delete contents[slot];
     name[slot] = newName;
     contents[slot] = newContents;
     newContents = 0;
+    lock.Lock();
+    ++data.started;
+    ++data.in_flight;
   }
+  lock.Unlock();
 
   while (data.finished < data.started) {
     slot = data.finished % concurrentios;
@@ -430,7 +438,7 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
       lock.Unlock();
       goto ERR;
     }
-    data.cur_latency = ceph_clock_now(g_ceph_context) - start_times[slot];
+    data.cur_latency = ceph_clock_now(cct) - start_times[slot];
     data.history.latency.push_back(data.cur_latency);
     total_latency += data.cur_latency;
     if (data.cur_latency > data.max_latency) data.max_latency = data.cur_latency;
@@ -443,7 +451,7 @@ int ObjBencher::write_bench(int secondsToRun, int concurrentios) {
     delete contents[slot];
   }
 
-  timePassed = ceph_clock_now(g_ceph_context) - data.start_time;
+  timePassed = ceph_clock_now(cct) - data.start_time;
   lock.Lock();
   data.done = true;
   lock.Unlock();
@@ -521,7 +529,7 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
 
   lock.Lock();
   data.finished = 0;
-  data.start_time = ceph_clock_now(g_ceph_context);
+  data.start_time = ceph_clock_now(cct);
   lock.Unlock();
 
   pthread_t print_thread;
@@ -531,7 +539,7 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
   //start initial reads
   for (int i = 0; i < concurrentios; ++i) {
     index[i] = i;
-    start_times[i] = ceph_clock_now(g_ceph_context);
+    start_times[i] = ceph_clock_now(cct);
     create_completion(i, _aio_cb, (void *)&lc);
     r = aio_read(name[i], i, contents[i], data.object_size);
     if (r < 0) { //naughty, doesn't clean up heap -- oh, or handle the print thread!
@@ -549,7 +557,7 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
   bufferlist *cur_contents;
 
   slot = 0;
-  while (seconds_to_run && (ceph_clock_now(g_ceph_context) < finish_time) &&
+  while (seconds_to_run && (ceph_clock_now(cct) < finish_time) &&
       num_objects > data.started) {
     lock.Lock();
     int old_slot = slot;
@@ -582,7 +590,7 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
       lock.Unlock();
       goto ERR;
     }
-    data.cur_latency = ceph_clock_now(g_ceph_context) - start_times[slot];
+    data.cur_latency = ceph_clock_now(cct) - start_times[slot];
     total_latency += data.cur_latency;
     if( data.cur_latency > data.max_latency) data.max_latency = data.cur_latency;
     if (data.cur_latency < data.min_latency) data.min_latency = data.cur_latency;
@@ -594,7 +602,7 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
     cur_contents = contents[slot];
 
     //start new read and check data if requested
-    start_times[slot] = ceph_clock_now(g_ceph_context);
+    start_times[slot] = ceph_clock_now(cct);
     contents[slot] = new bufferlist();
     create_completion(slot, _aio_cb, (void *)&lc);
     r = aio_read(newName, slot, contents[slot], data.object_size);
@@ -625,7 +633,7 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
       lock.Unlock();
       goto ERR;
     }
-    data.cur_latency = ceph_clock_now(g_ceph_context) - start_times[slot];
+    data.cur_latency = ceph_clock_now(cct) - start_times[slot];
     total_latency += data.cur_latency;
     if (data.cur_latency > data.max_latency) data.max_latency = data.cur_latency;
     if (data.cur_latency < data.min_latency) data.min_latency = data.cur_latency;
@@ -642,7 +650,7 @@ int ObjBencher::seq_read_bench(int seconds_to_run, int num_objects, int concurre
     delete contents[slot];
   }
 
-  runtime = ceph_clock_now(g_ceph_context) - data.start_time;
+  runtime = ceph_clock_now(cct) - data.start_time;
   lock.Lock();
   data.done = true;
   lock.Unlock();
