@@ -194,21 +194,11 @@ int rgw_bucket_instance_remove_entry(RGWRados *store, string& entry, RGWObjVersi
   return store->meta_mgr->remove_entry(bucket_instance_meta_handler, entry, objv_tracker);
 }
 
-int rgw_bucket_set_attrs(RGWRados *store, RGWBucketInfo& bucket_info,
+int rgw_bucket_set_attrs(RGWRados *store, rgw_bucket& bucket,
                          map<string, bufferlist>& attrs,
                          map<string, bufferlist>* rmattrs,
                          RGWObjVersionTracker *objv_tracker)
 {
-  rgw_bucket& bucket = bucket_info.bucket;
-
-  if (!bucket_info.has_instance_obj) {
-    /* an old bucket object, need to convert it */
-    int ret = store->convert_old_bucket_info(NULL, bucket.name);
-    if (ret < 0) {
-      ldout(store->ctx(), 0) << "ERROR: failed converting old bucket info: " << ret << dendl;
-      return ret;
-    }
-  }
   string oid;
   store->get_bucket_meta_oid(bucket, oid);
   rgw_obj obj(store->zone.domain_root, oid);
@@ -333,7 +323,7 @@ int rgw_remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_children)
 
   if (delete_children) {
     int max = 1000;
-    ret = store->list_objects(bucket, max, prefix, delim, marker, NULL,
+    ret = store->list_objects(bucket, max, prefix, delim, marker,
             objs, common_prefixes,
             false, ns, true, NULL, NULL);
 
@@ -349,7 +339,7 @@ int rgw_remove_bucket(RGWRados *store, rgw_bucket& bucket, bool delete_children)
       }
       objs.clear();
 
-      ret = store->list_objects(bucket, max, prefix, delim, marker, NULL, objs, common_prefixes,
+      ret = store->list_objects(bucket, max, prefix, delim, marker, objs, common_prefixes,
                                 false, ns, true, NULL, NULL);
       if (ret < 0)
         return ret;
@@ -597,7 +587,7 @@ int RGWBucket::check_bad_index_multipart(RGWBucketAdminOpState& op_state,
 
   do {
     vector<RGWObjEnt> result;
-    int r = store->list_objects(bucket, max, prefix, delim, marker, NULL,
+    int r = store->list_objects(bucket, max, prefix, delim, marker,
                                 result, common_prefixes, false,
                                 ns, true,
                                 &is_truncated, NULL);
@@ -727,9 +717,17 @@ int RGWBucket::check_index(RGWBucketAdminOpState& op_state,
   return 0;
 }
 
-
-int RGWBucket::policy_bl_to_stream(bufferlist& bl, ostream& o)
+int RGWBucket::get_policy(RGWBucketAdminOpState& op_state, ostream& o)
 {
+  std::string object_name = op_state.get_object_name();
+  rgw_bucket bucket = op_state.get_bucket();
+
+  bufferlist bl;
+  rgw_obj obj(bucket, object_name);
+  int ret = store->get_attr(NULL, obj, RGW_ATTR_ACL, bl);
+  if (ret < 0)
+    return ret;
+
   RGWAccessControlPolicy_S3 policy(g_ceph_context);
   bufferlist::iterator iter = bl.begin();
   try {
@@ -739,40 +737,10 @@ int RGWBucket::policy_bl_to_stream(bufferlist& bl, ostream& o)
     return -EIO;
   }
   policy.to_xml(o);
+
   return 0;
 }
 
-int RGWBucket::get_policy(RGWBucketAdminOpState& op_state, ostream& o)
-{
-  std::string object_name = op_state.get_object_name();
-  rgw_bucket bucket = op_state.get_bucket();
-
-  if (!object_name.empty()) {
-    bufferlist bl;
-    rgw_obj obj(bucket, object_name);
-    int ret = store->get_attr(NULL, obj, RGW_ATTR_ACL, bl);
-    if (ret < 0)
-      return ret;
-
-    return policy_bl_to_stream(bl, o);
-  }
-
-
-  RGWBucketInfo bucket_info;
-  map<string, bufferlist> attrs;
-  int ret = store->get_bucket_info(NULL, bucket.name, bucket_info, NULL, &attrs);
-  if (ret < 0) {
-    return ret;
-  }
-
-  map<string, bufferlist>::iterator aiter = attrs.find(RGW_ATTR_ACL);
-  if (aiter == attrs.end()) {
-    return -ENOENT;
-  }
-
-  return policy_bl_to_stream(aiter->second, o);
-}
-
 
 int RGWBucketAdminOp::get_policy(RGWRados *store, RGWBucketAdminOpState& op_state,
                   ostream& os)
@@ -933,6 +901,7 @@ static int bucket_stats(RGWRados *store, std::string&  bucket_name, Formatter *f
   formatter->dump_int("mtime", mtime);
   formatter->dump_string("max_marker", max_marker);
   dump_bucket_usage(stats, formatter);
+  encode_json("bucket_quota", bucket_info.quota, formatter);
   formatter->close_section();
 
   return 0;
