@@ -2,12 +2,13 @@
 #include <stdlib.h>
 #include <unistd.h>
 
-#include "rgw_json.h"
+#include "common/ceph_json.h"
 #include "rgw_common.h"
 #include "rgw_swift.h"
 #include "rgw_swift_auth.h"
 #include "rgw_user.h"
 #include "rgw_http_client.h"
+#include "rgw_keystone.h"
 
 #include "include/str_list.h"
 
@@ -18,23 +19,26 @@
 
 static list<string> roles_list;
 
-class RGWKeystoneTokenCache;
-
 class RGWValidateSwiftToken : public RGWHTTPClient {
-  CephContext *cct;
   struct rgw_swift_auth_info *info;
 
 protected:
-  RGWValidateSwiftToken() : cct(NULL), info(NULL) {}
+  RGWValidateSwiftToken() : RGWHTTPClient(NULL), info(NULL) {}
 public:
-  RGWValidateSwiftToken(CephContext *_cct, struct rgw_swift_auth_info *_info) : cct(_cct), info(_info) {}
+  RGWValidateSwiftToken(CephContext *_cct, struct rgw_swift_auth_info *_info) : RGWHTTPClient(_cct), info(_info) {}
 
-  int read_header(void *ptr, size_t len);
+  int receive_header(void *ptr, size_t len);
+  int receive_data(void *ptr, size_t len) {
+    return 0;
+  }
+  int send_data(void *ptr, size_t len) {
+    return 0;
+  }
 
   friend class RGWKeystoneTokenCache;
 };
 
-int RGWValidateSwiftToken::read_header(void *ptr, size_t len)
+int RGWValidateSwiftToken::receive_header(void *ptr, size_t len)
 {
   char line[len + 1];
 
@@ -100,202 +104,24 @@ int RGWSwift::validate_token(const char *token, struct rgw_swift_auth_info *info
   return 0;
 }
 
-int KeystoneToken::parse(CephContext *cct, bufferlist& bl)
-{
-  RGWJSONParser parser;
 
-  if (!parser.parse(bl.c_str(), bl.length())) {
-    ldout(cct, 0) << "malformed json" << dendl;
-    return -EINVAL;
-  }
-
-  JSONObjIter iter = parser.find_first("access");
-  if (iter.end()) {
-    ldout(cct, 0) << "token response is missing access section" << dendl;
-    return -EINVAL;
-  }  
-
-  JSONObj *access_obj = *iter;
-  JSONObj *user = access_obj->find_obj("user");
-  if (!user) {
-    ldout(cct, 0) << "token response is missing user section" << dendl;
-    return -EINVAL;
-  }
-
-  if (!user->get_data("username", &user_name)) {
-    ldout(cct, 0) << "token response is missing user username field" << dendl;
-    return -EINVAL;
-  }
-
-  JSONObj *roles_obj = user->find_obj("roles");
-  if (!roles_obj) {
-    ldout(cct, 0) << "token response is missing roles section, or section empty" << dendl;
-    return -EINVAL;
-  }
-
-  JSONObjIter riter = roles_obj->find_first();
-  if (riter.end()) {
-    ldout(cct, 0) << "token response has an empty roles list" << dendl;
-    return -EINVAL;
-  }
-
-  for (; !riter.end(); ++riter) {
-    JSONObj *role_obj = *riter;
-    if (!role_obj) {
-      ldout(cct, 0) << "ERROR: role object is NULL" << dendl;
-      return -EINVAL;
-    }
-
-    JSONObj *role_name = role_obj->find_obj("name");
-    if (!role_name) {
-      ldout(cct, 0) << "token response is missing role name section" << dendl;
-      return -EINVAL;
-    }
-    string role = role_name->get_data();
-    roles[role] = true;
-  }
-
-  JSONObj *token = access_obj->find_obj("token");
-  if (!token) {
-    ldout(cct, 0) << "missing token section in response" << dendl;
-    return -EINVAL;
-  }
-
-  string expires;
-
-  if (!token->get_data("expires", &expires)) {
-    ldout(cct, 0) << "token response is missing expiration field" << dendl;
-    return -EINVAL;
-  }
-
-  struct tm t;
-  if (!parse_iso8601(expires.c_str(), &t)) {
-    ldout(cct, 0) << "failed to parse token expiration (" << expires << ")" << dendl;
-    return -EINVAL;
-  }
-
-  expiration = timegm(&t);
-
-  JSONObj *tenant = token->find_obj("tenant");
-  if (!tenant) {
-    ldout(cct, 0) << "token response is missing tenant section" << dendl;
-    return -EINVAL;
-  }
-
-  if (!tenant->get_data("id", &tenant_id)) {
-    ldout(cct, 0) << "tenant is missing id field" << dendl;
-    return -EINVAL;
-  }
-
-
-  if (!tenant->get_data("name", &tenant_name)) {
-    ldout(cct, 0) << "tenant is missing name field" << dendl;
-    return -EINVAL;
-  }
-
-  return 0;
-}
-
-struct token_entry {
-  KeystoneToken token;
-  list<string>::iterator lru_iter;
-};
-
-class RGWKeystoneTokenCache {
-  CephContext *cct;
-
-  map<string, token_entry> tokens;
-  list<string> tokens_lru;
-
-  Mutex lock;
-
-  size_t max;
-
-public:
-  RGWKeystoneTokenCache(CephContext *_cct, int _max) : cct(_cct), lock("RGWKeystoneTokenCache"), max(_max) {}
-
-  bool find(const string& token_id, KeystoneToken& token);
-  void add(const string& token_id, KeystoneToken& token);
-  void invalidate(const string& token_id);
-};
-
-bool RGWKeystoneTokenCache::find(const string& token_id, KeystoneToken& token)
-{
-  lock.Lock();
-  map<string, token_entry>::iterator iter = tokens.find(token_id);
-  if (iter == tokens.end()) {
-    lock.Unlock();
-    if (perfcounter) perfcounter->inc(l_rgw_keystone_token_cache_miss);
-    return false;
-  }
-
-  token_entry& entry = iter->second;
-  tokens_lru.erase(entry.lru_iter);
-
-  if (entry.token.expired()) {
-    tokens.erase(iter);
-    lock.Unlock();
-    if (perfcounter) perfcounter->inc(l_rgw_keystone_token_cache_hit);
-    return false;
-  }
-  token = entry.token;
-
-  tokens_lru.push_front(token_id);
-  entry.lru_iter = tokens_lru.begin();
-
-  lock.Unlock();
-  if (perfcounter) perfcounter->inc(l_rgw_keystone_token_cache_hit);
-
-  return true;
-}
-
-void RGWKeystoneTokenCache::add(const string& token_id, KeystoneToken& token)
-{
-  lock.Lock();
-  map<string, token_entry>::iterator iter = tokens.find(token_id);
-  if (iter != tokens.end()) {
-    token_entry& e = iter->second;
-    tokens_lru.erase(e.lru_iter);
-  }
-
-  tokens_lru.push_front(token_id);
-  token_entry& entry = tokens[token_id];
-  entry.token = token;
-  entry.lru_iter = tokens_lru.begin();
-
-  while (tokens_lru.size() > max) {
-    list<string>::reverse_iterator riter = tokens_lru.rbegin();
-    iter = tokens.find(*riter);
-    assert(iter != tokens.end());
-    tokens.erase(iter);
-    tokens_lru.pop_back();
-  }
-  
-  lock.Unlock();
-}
-
-void RGWKeystoneTokenCache::invalidate(const string& token_id)
-{
-  Mutex::Locker l(lock);
-  map<string, token_entry>::iterator iter = tokens.find(token_id);
-  if (iter == tokens.end())
-    return;
-
-  ldout(cct, 20) << "invalidating revoked token id=" << token_id << dendl;
-  token_entry& e = iter->second;
-  tokens_lru.erase(e.lru_iter);
-  tokens.erase(iter);
-}
 
 class RGWValidateKeystoneToken : public RGWHTTPClient {
   bufferlist *bl;
 public:
-  RGWValidateKeystoneToken(bufferlist *_bl) : bl(_bl) {}
+  RGWValidateKeystoneToken(CephContext *_cct, bufferlist *_bl) : RGWHTTPClient(_cct), bl(_bl) {}
 
-  int read_data(void *ptr, size_t len) {
+  int receive_data(void *ptr, size_t len) {
     bl->append((char *)ptr, len);
     return 0;
   }
+  int receive_header(void *ptr, size_t len) {
+    return 0;
+  }
+  int send_data(void *ptr, size_t len) {
+    return 0;
+  }
+
 };
 
 static RGWKeystoneTokenCache *keystone_token_cache = NULL;
@@ -303,12 +129,18 @@ static RGWKeystoneTokenCache *keystone_token_cache = NULL;
 class RGWGetRevokedTokens : public RGWHTTPClient {
   bufferlist *bl;
 public:
-  RGWGetRevokedTokens(bufferlist *_bl) : bl(_bl) {}
+  RGWGetRevokedTokens(CephContext *_cct, bufferlist *_bl) : RGWHTTPClient(_cct), bl(_bl) {}
 
-  int read_data(void *ptr, size_t len) {
+  int receive_data(void *ptr, size_t len) {
     bl->append((char *)ptr, len);
     return 0;
   }
+  int receive_header(void *ptr, size_t len) {
+    return 0;
+  }
+  int send_data(void *ptr, size_t len) {
+    return 0;
+  }
 };
 
 static int open_cms_envelope(CephContext *cct, string& src, string& dst)
@@ -383,7 +215,7 @@ static int decode_b64_cms(CephContext *cct, const string& signed_b64, bufferlist
 int RGWSwift::check_revoked()
 {
   bufferlist bl;
-  RGWGetRevokedTokens req(&bl);
+  RGWGetRevokedTokens req(cct, &bl);
 
   string url = g_conf->rgw_keystone_url;
   if (url.empty()) {
@@ -396,7 +228,7 @@ int RGWSwift::check_revoked()
 
   req.append_header("X-Auth-Token", g_conf->rgw_keystone_admin_token);
 
-  int ret = req.process(url);
+  int ret = req.process(url.c_str());
   if (ret < 0)
     return ret;
 
@@ -404,7 +236,7 @@ int RGWSwift::check_revoked()
 
   ldout(cct, 10) << "request returned " << bl.c_str() << dendl;
 
-  RGWJSONParser parser;
+  JSONParser parser;
 
   if (!parser.parse(bl.c_str(), bl.length())) {
     ldout(cct, 0) << "malformed json" << dendl;
@@ -438,7 +270,7 @@ int RGWSwift::check_revoked()
 
   ldout(cct, 10) << "ceph_decode_cms: decoded: " << json.c_str() << dendl;
 
-  RGWJSONParser list_parser;
+  JSONParser list_parser;
   if (!list_parser.parse(json.c_str(), json.length())) {
     ldout(cct, 0) << "malformed json" << dendl;
     return -EINVAL;
@@ -471,8 +303,8 @@ int RGWSwift::check_revoked()
 
 static void rgw_set_keystone_token_auth_info(KeystoneToken& token, struct rgw_swift_auth_info *info)
 {
-  info->user = token.tenant_id;
-  info->display_name = token.tenant_name;
+  info->user = token.token.tenant.id;
+  info->display_name = token.token.tenant.name;
   info->status = 200;
 }
 
@@ -486,10 +318,8 @@ int RGWSwift::parse_keystone_token_response(const string& token, bufferlist& bl,
   list<string>::iterator iter;
   for (iter = roles_list.begin(); iter != roles_list.end(); ++iter) {
     const string& role = *iter;
-    if (t.roles.find(role) != t.roles.end()) {
-      found = true;
+    if ((found=t.user.has_role(role))==true)
       break;
-    }
   }
 
   if (!found) {
@@ -497,7 +327,7 @@ int RGWSwift::parse_keystone_token_response(const string& token, bufferlist& bl,
     return -EPERM;
   }
 
-  ldout(cct, 0) << "validated token: " << t.tenant_name << ":" << t.user_name << " expires: " << t.expiration << dendl;
+  ldout(cct, 0) << "validated token: " << t.token.tenant.name << ":" << t.user.name << " expires: " << t.token.expires << dendl;
 
   rgw_set_keystone_token_auth_info(t, info);
 
@@ -511,7 +341,7 @@ int RGWSwift::update_user_info(RGWRados *store, struct rgw_swift_auth_info *info
     user_info.user_id = info->user;
     user_info.display_name = info->display_name;
 
-    int ret = rgw_store_user_info(store, user_info, true);
+    int ret = rgw_store_user_info(store, user_info, NULL, NULL, 0, true);
     if (ret < 0) {
       ldout(cct, 0) << "ERROR: failed to store new user's info: ret=" << ret << dendl;
       return ret;
@@ -574,7 +404,7 @@ int RGWSwift::validate_keystone_token(RGWRados *store, const string& token, stru
   if (keystone_token_cache->find(token_id, t)) {
     rgw_set_keystone_token_auth_info(t, info);
 
-    ldout(cct, 20) << "cached token.tenant_id=" << t.tenant_id << dendl;
+    ldout(cct, 20) << "cached token.tenant.id=" << t.token.tenant.id << dendl;
 
     int ret = update_user_info(store, info, rgw_user);
     if (ret < 0)
@@ -590,7 +420,7 @@ int RGWSwift::validate_keystone_token(RGWRados *store, const string& token, stru
 
     /* can't decode, just go to the keystone server for validation */
 
-    RGWValidateKeystoneToken validate(&bl);
+    RGWValidateKeystoneToken validate(cct, &bl);
 
     string url = g_conf->rgw_keystone_url;
     if (url.empty()) {
@@ -604,7 +434,7 @@ int RGWSwift::validate_keystone_token(RGWRados *store, const string& token, stru
 
     validate.append_header("X-Auth-Token", g_conf->rgw_keystone_admin_token);
 
-    int ret = validate.process(url);
+    int ret = validate.process(url.c_str());
     if (ret < 0)
       return ret;
   }
@@ -644,14 +474,14 @@ bool RGWSwift::verify_swift_token(RGWRados *store, req_state *s)
 
   int ret;
 
-  if (g_conf->rgw_swift_use_keystone) {
+  if (supports_keystone()) {
     ret = validate_keystone_token(store, s->os_auth_token, &info, s->user);
     return (ret >= 0);
   }
 
   ret = validate_token(s->os_auth_token, &info);
   if (ret < 0)
-    return ret;
+    return false;
 
   if (info.user.empty()) {
     ldout(cct, 5) << "swift auth didn't authorize a user" << dendl;
@@ -678,7 +508,13 @@ bool RGWSwift::verify_swift_token(RGWRados *store, req_state *s)
 void RGWSwift::init()
 {
   get_str_list(cct->_conf->rgw_keystone_accepted_roles, roles_list);
+  if (supports_keystone())
+      init_keystone();
+}
 
+
+void RGWSwift::init_keystone()
+{
   keystone_token_cache = new RGWKeystoneTokenCache(cct, cct->_conf->rgw_keystone_token_cache_size);
 
   keystone_revoke_thread = new KeystoneRevokeThread(cct, this);
@@ -687,6 +523,12 @@ void RGWSwift::init()
 
 
 void RGWSwift::finalize()
+{
+  if (supports_keystone())
+    finalize_keystone();
+}
+
+void RGWSwift::finalize_keystone()
 {
   delete keystone_token_cache;
   keystone_token_cache = NULL;
